scriptname sslCreatureAnimationSlots extends sslAnimationSlots

; ------------------------------------------------------- ;
; --- Creature aniamtion support                      --- ;
; ------------------------------------------------------- ;

string function GetRaceKey(Race RaceRef) global native
string function GetRaceKeyByID(string RaceID) global native
function AddRaceID(string RaceKey, string RaceID) global native
bool function HasRaceID(string RaceKey, string RaceID) global native
bool function HasRaceKey(string RaceKey) global native
bool function ClearRaceKey(string RaceKey) global native
bool function HasRaceIDType(string RaceID) global native
bool function HasCreatureType(Actor ActorRef) global native
bool function HasRaceType(Race RaceRef) global native
string[] function GetAllRaceKeys(Race RaceRef = none) global native
string[] function GetAllRaceIDs(string RaceKey) global native
Race[] function GetAllRaces(string RaceKey) global native

import PapyrusUtil

sslBaseAnimation[] function GetByRace(int ActorCount, Race RaceRef)
	Log("GetByRace(ActorCount="+ActorCount+", RaceRef="+RaceRef+")")
	string[] RaceTypes = GetAllRaceKeys(RaceRef)
	if RaceTypes.Length < 1
		return sslUtility.AnimationArray(0)
	endIf
	bool[] Valid  = Utility.CreateBoolArray(Slotted)
	int i = Slotted
	while i
		i -= 1
		sslBaseAnimation Slot = GetBySlot(i)
		Valid[i] = Slot.Enabled && ActorCount == Slot.PositionCount && RaceTypes.Find(Slot.RaceType) != -1
	endWhile
	return GetList(Valid)
endFunction

sslBaseAnimation[] function GetByRaceTags(int ActorCount, Race RaceRef, string Tags, string TagsSuppressed = "", bool RequireAll = true)
	Log("GetByRaceTags(ActorCount="+ActorCount+", RaceRef="+RaceRef+", Tags="+Tags+", TagsSuppressed="+TagsSuppressed+", RequireAll="+RequireAll+")")
	string[] RaceTypes = GetAllRaceKeys(RaceRef)
	if RaceTypes.Length < 1
		return sslUtility.AnimationArray(0)
	endIf
	bool[] Valid      = Utility.CreateBoolArray(Slotted)
	string[] Suppress = StringSplit(TagsSuppressed)
	string[] Search   = StringSplit(Tags)
	int i = Slotted
	while i
		i -= 1
		sslBaseAnimation Slot = GetBySlot(i)
		Valid[i] = Slot && Slot.Enabled && RaceTypes.Find(Slot.RaceType) != -1 && ActorCount == Slot.PositionCount && Slot.TagSearch(Search, Suppress, RequireAll)
	endWhile
	return GetList(Valid)
endFunction

sslBaseAnimation[] function GetByRaceKey(int ActorCount, string RaceKey)
	Log("GetByRaceKey(ActorCount="+ActorCount+", RaceKey="+RaceKey+")")
	if !HasRaceKey(RaceKey)
		return sslUtility.AnimationArray(0)
	endIf
	bool[] Valid  = Utility.CreateBoolArray(Slotted)
	int i = Slotted
	while i
		i -= 1
		sslBaseAnimation Slot = GetBySlot(i)
		Valid[i] = Slot && Slot.RaceType == RaceKey && Slot.Enabled && ActorCount == Slot.PositionCount
	endWhile
	return GetList(Valid)
endFunction

sslBaseAnimation[] function GetByRaceKeyTags(int ActorCount, string RaceKey, string Tags, string TagsSuppressed = "", bool RequireAll = true)
	Log("GetByRaceKeyTags(ActorCount="+ActorCount+", RaceKey="+RaceKey+", Tags="+Tags+", TagsSuppressed="+TagsSuppressed+", RequireAll="+RequireAll+")")
	if !HasRaceKey(RaceKey)
		return sslUtility.AnimationArray(0)
	endIf
	bool[] Valid      = Utility.CreateBoolArray(Slotted)
	string[] Suppress = StringSplit(TagsSuppressed)
	string[] Search   = StringSplit(Tags)
	int i = Slotted
	while i
		i -= 1
		sslBaseAnimation Slot = GetBySlot(i)
		Valid[i] = Slot && Slot.RaceType == RaceKey && Slot.Enabled && ActorCount == Slot.PositionCount && Slot.TagSearch(Search, Suppress, RequireAll)
	endWhile
	return GetList(Valid)
endFunction

sslBaseAnimation[] function GetByRaceGenders(int ActorCount, Race RaceRef, int MaleCreatures = 0, int FemaleCreatures = 0, bool ForceUse = false)
	Log("GetByRaceGenders(ActorCount="+ActorCount+", RaceRef="+RaceRef+", MaleCreatures="+MaleCreatures+", FemaleCreatures="+FemaleCreatures+", ForceUse="+ForceUse+")")
	if !Config.UseCreatureGender && !ForceUse
		return GetByRace(ActorCount, RaceRef)
	endIf
	string[] RaceTypes = GetAllRaceKeys(RaceRef)
	if RaceTypes.Length < 1
		return sslUtility.AnimationArray(0)
	endIf
	bool[] Valid  = Utility.CreateBoolArray(Slotted)
	int i = Slotted
	while i
		i -= 1
		sslBaseAnimation Slot = GetBySlot(i)
		Valid[i] = Slot && Slot.Enabled && RaceTypes.Find(Slot.RaceType) != -1 && ActorCount == Slot.PositionCount && (!Slot.GenderedCreatures || (MaleCreatures == Slot.MaleCreatures && FemaleCreatures == Slot.FemaleCreatures))
	endWhile
	return GetList(Valid)
endFunction

sslBaseAnimation[] function FilterCreatureGenders(sslBaseAnimation[] Anims, int MaleCreatures = 0, int FemaleCreatures = 0)
	if !Config.UseCreatureGender || !Anims
		return Anims
	endIf
	int Del
	int i = Anims.Length
	while i
		i -= 1
		if Anims[i] && Anims[i].GenderedCreatures && (MaleCreatures != Anims[i].MaleCreatures || FemaleCreatures != Anims[i].FemaleCreatures)
			Anims[i] = none
			Del += 1
		endIf
	endWhile
	if Del == 0
		return Anims
	endIf
	i = Anims.Length
	int n = (i - Del)
	sslBaseAnimation[] Output = sslUtility.AnimationArray(n)
	while i && n
		i -= 1
		if Anims[i] != none
			n -= 1
			Output[n] = Anims[i]
		endIf
	endWhile
	return Output
endFunction

bool function RaceHasAnimation(Race RaceRef, int ActorCount = -1, int Gender = -1)
	string[] RaceTypes = GetAllRaceKeys(RaceRef)
	if RaceTypes
		bool UseGender = Gender != -1 && Config.UseCreatureGender
		int i = Slotted
		while i
			i -= 1
			sslBaseAnimation Slot = GetBySlot(i)
			if Slot && Slot.Enabled && RaceTypes.Find(Slot.RaceType) != -1 && (ActorCount == -1 || ActorCount == Slot.PositionCount) && ((!UseGender || !Slot.GenderedCreatures) || Slot.Genders[Gender] > 0)
				return true
			endIf
		endWhile
	endIf
	return false
endFunction

bool function RaceKeyHasAnimation(string RaceKey, int ActorCount = -1, int Gender = -1)
	if HasRaceKey(RaceKey)
		bool UseGender = Gender != -1 && Config.UseCreatureGender
		int i = Slotted
		while i
			i -= 1
			sslBaseAnimation Slot = GetBySlot(i)
			if Slot && Slot.Enabled && RaceKey == Slot.RaceType && (ActorCount == -1 || ActorCount == Slot.PositionCount) && ((!UseGender || !Slot.GenderedCreatures) || Slot.Genders[Gender] > 0)
				return true
			endIf
		endWhile
	endIf
	return false
endFunction

bool function HasCreature(Actor ActorRef)
	return sslCreatureAnimationSlots.HasCreatureType(ActorRef)
endFunction

bool function HasRace(Race RaceRef)
	return sslCreatureAnimationSlots.HasRaceType(RaceRef)
endFunction

bool function AllowedCreature(Race RaceRef)
	return Config.AllowCreatures && HasAnimation(RaceRef)
endFunction

bool function AllowedCreatureCombination(Race RaceRef1, Race RaceRef2)
	return Config.AllowCreatures && sslCreatureAnimationSlots.GetRaceKey(RaceRef1) == sslCreatureAnimationSlots.GetRaceKey(RaceRef2)
endFunction

; Deprecated
bool function HasAnimation(Race RaceRef, int Gender = -1)
	return RaceHasAnimation(RaceRef, -1, Gender)
endFunction

; ------------------------------------------------------- ;
; --- System Use Only                                 --- ;
; ------------------------------------------------------- ;

function Setup()
	RegisterRaces()
	parent.Setup()
endfunction

function RegisterSlots()
	if Config.AllowCreatures
		; Register the creature animations and voices
		PreloadCategoryLoaders()
		(Game.GetFormFromFile(0x664FB, "SexLab.esm") as sslCreatureAnimationDefaults).LoadCreatureAnimations()
		(Game.GetFormFromFile(0x664FB, "SexLab.esm") as sslVoiceDefaults).LoadCreatureVoices()
		ModEvent.Send(ModEvent.Create("SexLabSlotCreatureAnimations"))
		Debug.Notification("$SSL_NotifyCreatureAnimationInstall")
	else
		Config.Log("Creatures not enabled, skipping registration.", "RegisterSlots() Creature")
	endIf
endFunction

function RegisterRaces()
	; 2017.04.06 / Add
	self.addRacesFromFile("Bears");
	self.addRacesFromFile("SabreCats");
	self.addRacesFromFile("Chaurus");
	self.addRacesFromFile("Dragons");
	self.addRacesFromFile("Draugrs");
	self.addRacesFromFile("Falmers");
	self.addRacesFromFile("Giants");
	self.addRacesFromFile("Horses", "Horseses");
	self.addRacesFromFile("Spiders");
	self.addRacesFromFile("LargeSpiders");
	self.addRacesFromFile("Trolls");
    self.addRacesFromFile("Werewolves", "Werewolfpanic");
    self.addRacesFromFile("Wolves", "Wolfpanic");
    self.addRacesFromFile("Dogs", "Dogpanic");
	self.addRacesFromFile("VampireLords");
	self.addRacesFromFile("Gargoyles");
	self.addRacesFromFile("Rieklings");
	self.addRacesFromFile("Seekers");
	self.addRacesFromFile("Lurkers");
    self.addRacesFromFile("Spriggans", "SprigganXan");
    self.addRacesFromFile("FlameAtronach");
    self.addRacesFromFile("Skeevers");
    self.addRacesFromFile("Chickens");
	; 2017.04.06 / Add

	; Send creature race key registration event
	ModEvent.Send(ModEvent.Create("SexLabRegisterCreatureKey"))
endFunction

; 2017.04.06 / Add
Int Function addRacesFromFile(String szKey = "", String szAltKey = "")
    ;-------------------------------------------------------
    ; Init Variables
    ;-------------------------------------------------------
    String  szJson  = "../SexLab/CreatureRaces.json";
    Int     nIdx    = 0;
    Int     nCnt    = 0;
    Int     nLen    = 0;
    String  szRace  = "";

    ;-------------------------------------------------------
    ; Checking Json
    ;-------------------------------------------------------
    If !JsonUtil.IsGood(szJson)
        Debug.Notification("Syntax Error: " + szJson);
        return nCnt;
    Elseif (szKey == "")
        return nCnt;
    Elseif (szAltKey == "")
        szAltKey = szKey;
    Endif

    ;-------------------------------------------------------
    ; Clear Race Key
    ;-------------------------------------------------------
    ClearRaceKey(szKey);

    ;-------------------------------------------------------
    ; Loop of StringList
    ;-------------------------------------------------------
    nLen = JsonUtil.StringListCount(szJson, szAltKey);

    While (nIdx < nLen)
        szRace = \
            JsonUtil.StringListGet(szJson, szAltKey, nIdx);

        If (szRace != "")
            AddRaceID(szKey, szRace);
            nCnt += 1;
        Endif

        nIdx += 1;
    EndWhile

    ;-------------------------------------------------------
    ; Return
    ;-------------------------------------------------------
    return nCnt;
EndFunction
; 2017.04.06 / Add
